<?php

/**
 *
 * @package   Woocommerce JCC Payment Gateway
 * @category Payment Gateway
 * @author   JCC
 */
if (!class_exists('WooCommerce_JCC_Payment_Gateway')) :

    class WooCommerce_JCC_Payment_Gateway extends WC_Payment_Gateway {

        /**
         * Init and hook in the payment_gateway.
         */
        public function __construct() {
            global $woocommerce;
            $this->id = 'jcc_payment_gateway';
            $this->method_title = __('JCC Payment Gateway', 'woocommerce_jcc_payment_gateway');
            $this->method_description = __('JCC’s payment gateway offers real-time '
                    . 'and batch payment processing. It uses all available '
                    . 'security measures to prevent fraudulent transactions and '
                    . 'ensure data safety yet it’s easy to integrate with '
                    . 'merchants’ systems. In addition, it allows merchants to '
                    . 'review and manage transactions, prepare reports, etc. '
                    . 'through a user-friendly, intuitive administration interface. '
                    . 'Another feature the plugin offers, is the ability for the merchant to define a prefix value that will be appended in the merchant order id that is sent to JCCgateway. '
                    . 'The current plugin supports making payment via HTTP Post '
                    . 'redirect to JCC payment gateway and also refunds via JCC '
                    . 'Web Services\'s endpoint, called Financial Service.', 'woocommerce_jcc_payment_gateway');
            $this->icon = apply_filters('woocommerce_jcc_gateway_checkout_icon', plugins_url('../assets/img/JCCgateway_checkout.jpg', __FILE__) );
            $this->has_fields = false;
            $this->order_button_text = __('Procceed to JCC Gateway ', 'woocommerce_jcc_payment_gateway');
            $this->notify_url = WC()->api_request_url('WooCommerce_JCC_Payment_Gateway');

            $this->supports = array(
                'products',
                'refunds');

            // Load the settings.
            $this->init_form_fields();
            $this->init_settings();
             
            $this->title= $this->get_option( 'title' );

            // Define user set variables
            $this->test_mode = $this->get_option('test_mode');
            // Test configurations
            $this->test_financial_service_wsdl = $this->get_option('test_financial_service_wsdl');
            $this->test_request_url = $this->get_option('test_request_url');
            $this->test_merchant_id = $this->get_option('test_merchant_id');
            $this->test_password = $this->get_option('test_password');
            // Production configurations
            $this->production_financial_service_wsdl = $this->get_option('production_financial_service_wsdl');
            $this->production_request_url = $this->get_option('production_request_url');
            $this->production_merchant_id = $this->get_option('production_merchant_id');
            $this->production_password = $this->get_option('production_password');
            // General configurations
            $this->merchant_order_id_prefix = $this->get_option('merchant_order_id_prefix');
            $this->version = $this->get_option('version');
            $this->acquirer_id = $this->get_option('acquirer_id');
            $this->capture_flag = $this->get_option('capture_flag');
            $this->signature_method = $this->get_option('signature_method');

            // Actions
            //add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_receipt_jcc_payment_gateway', array($this, 'receipt_jcc_payment_gateway'));
            // Payment listener/API hook
            add_action('woocommerce_api_woocommerce_jcc_payment_gateway', array($this, 'jcc_payment_gateway_response'));

            // Check if the gateway can be used
            if (!$this->is_valid_for_use()) {
                $this->enabled = false;
            }
        }

        /**
         * Check if the store currency is set to 'EUR', 'USD', 'GBP', 'CHF' or 'RUB'
         * */
        public function is_valid_for_use() {
            $is_valid_for_use = false;
            
            if (!in_array(get_woocommerce_currency(), array('EUR', 'USD', 'GBP', 'CHF', 'RUB'))) {
                $this->msg = __('JCC doesn\'t support your store currency, set it either to EURO (EUR), United States Dollar (USD), Greate Britain Pound (GBP), Swiss Franc (CHF), Russinan Rubble (RUB) <a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wc-settings&tab=general">here</a>', 'woocommerce_jcc_payment_gateway');
                $is_valid_for_use = false;
            }
            if (intval(wc_get_price_decimals()) != 2) {
                $this->msg = 'JCC doesn\'t support your store currency number of decimals, set it 2. <a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wc-settings&tab=general">here</a>';
                $is_valid_for_use = false;
            }
            try {
                $wc_jcc_api = $this->get_wc_jcc_api_instance();
                //syslog(LOG_INFO, 'WooCommerce_JCC_Payment_Gateway::: is_valid_for_use: wc_jcc_api: ' . print_r($wc_jcc_api,true));
                $is_valid_for_use = true;
            } catch (Exception $exception) {
                WC_Admin_Settings::add_error(esc_html__($exception->getMessage()));
                $is_valid_for_use = false;
            }
            return $is_valid_for_use;
        }

        /**
         * Check if this gateway is enabled
         */
        public function is_available() {
            $is_available = ( 'yes' === $this->enabled );
            try {
                $wc_jcc_api = $this->get_wc_jcc_api_instance();
                $is_available = true;
            } catch (Exception $exception) {
                WC_Admin_Settings::add_error(esc_html__($exception->getMessage()));
                
                $is_available = false;
            }
            return $is_available;
        }

        /*
         * Returns an instance of WC_JCC_API
         */

        private function get_wc_jcc_api_instance($data_array = array()) {
            $wc_jcc_payment_gateway_id = $this->id;
            $wc_jcc_payment_gateway_settings = get_option('woocommerce_' . $wc_jcc_payment_gateway_id . '_settings');
            $wc_jcc_payment_gateway_settings['payment_gateway_response_url'] = $this->notify_url;
            $wc_jcc_payment_gateway_settings['currency'] = get_woocommerce_currency();
            $wc_jcc_payment_gateway_settings['currency_exp'] = wc_get_price_decimals();

            $data = array_merge($wc_jcc_payment_gateway_settings, $data_array);

            include_once 'class-wc-jcc-api.php';
            return new WC_JCC_API($data);
        }

        /**
         * Initialize payment gateway settings form fields.
         */
        public function init_form_fields() {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'checkbox',
                    'label' => 'Enable/Disable JCC Payment Gateway',
                    'description' => __('Enable or disable the gateway.', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __('Title', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'Credit/Debit card',
                    'css' => 'width:350px;',
                ),
                'test_mode' => array(
                    'title' => __('Test Mode', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'checkbox',
                    'label' => 'Enable Test Mode',
                    'default' => 'yes',
                    'description' => __('Test mode enables you to test JCC payment gateway before going live.', 'woocommerce_jcc_payment_gateway'),
                ),
                'test_financial_service_wsdl' => array(
                    'title' => __('Test Financial Service WSDL', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Enter Test Financial Service WSDL, check JCC Payment Gateway – Developer’s Guide', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'https://tjccpg.jccsecure.com/PgWebService/services/FinancialService?wsdl',
                    'css' => 'width:350px;',
                ),
                'test_request_url' => array(
                    'title' => __('Test Request URL', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Enter Test Request URL, check JCC Payment Gateway – Developer’s Guide', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'https://tjccpg.jccsecure.com/EcomPayment/RedirectAuthLink',
                    'css' => 'width:350px;',
                ),
                'test_merchant_id' => array(
                    'title' => __('Test Merchant ID', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Enter Test Merchant ID as provided by JCC', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => '',
                    'css' => 'width:350px;',
                ),
                'test_password' => array(
                    'title' => __('Test Password', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'password',
                    'description' => __('Enter Test Password as provided by JCC', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => '',
                    'css' => 'width:350px;',
                ),
                'production_financial_service_wsdl' => array(
                    'title' => __('Production Financial Service WSDL', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Enter Production Financial Service WSDL, check JCC Payment Gateway – Developer’s Guide', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'https://jccpg.jccsecure.com/PgWebService/services/FinancialService?wsdl',
                    'css' => 'width:350px;',
                ),
                'production_request_url' => array(
                    'title' => __('Production Request URL', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Enter Production Request URL, check JCC Payment Gateway – Developer’s Guide', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'https://jccpg.jccsecure.com/EcomPayment/RedirectAuthLink',
                    'css' => 'width:350px;',
                ),
                'production_merchant_id' => array(
                    'title' => __('Production Merchant ID', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Enter Production Merchant ID as provided by JCC', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => '',
                    'css' => 'width:350px;',
                ),
                'production_password' => array(
                    'title' => __('Production Password', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'password',
                    'description' => __('Enter Production Password as provided by JCC', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => '',
                    'css' => 'width:350px;',
                ),
                'merchant_order_id_prefix' => array(
                    'title' => __('Merchant Order ID Prefix', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Optional.Enter a prefix up to 10 characters.It will be appended to the order ID which is sent to JCC.'),
                    'desc_tip' => true,
                    'default' => '',
                    'css' => 'width:350px;',
                ),
                'version' => array(
                    'title' => __('Version', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Always the same, check JCC Payment Gateway – Developer’s Guide ', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => '1.0.0',
                    'css' => 'width:350px;',
                ),
                'acquirer_id' => array(
                    'title' => __('Acquirer ID', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'text',
                    'description' => __('Always the same, check JCC Payment Gateway – Developer’s Guide ', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => '',
                    'css' => 'width:350px;',
                ),
                'capture_flag' => array(
                    'title' => __('Capture Flag', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'select',
                    'description' => __('Specify we want not only to authorize the amount but also capture at the same time. Alternative value could be Manual (for capturing later)', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'A',
                    'options' => array(
                        'A' => __('Automatic', 'woocommerce_jcc_payment_gateway'),
                        'M' => __('Manual', 'woocommerce_jcc_payment_gateway'),
                    ),
                ),
                'signature_method' => array(
                    'title' => __('Signature Method', 'woocommerce_jcc_payment_gateway'),
                    'type' => 'select',
                    'description' => __('Hash algorithm used to create the signature; can be MD5 or SHA1. SHA1 is preferred.', 'woocommerce_jcc_payment_gateway'),
                    'desc_tip' => true,
                    'default' => 'SHA1',
                    'options' => array(
                        'SHA1' => __('SHA1', 'woocommerce_jcc_payment_gateway'),
                    ),
                ),
            );
        }

        public function process_payment($order_id) {
            global $woocommerce;
            $order = new WC_Order($order_id);
            if ($this->merchant_order_id_prefix){
              $order_key = $order->get_order_key();
              $order->set_order_key($this->merchant_order_id_prefix .'-'. $order_key);
              $order->save();
            }
            return array(
                'result' => 'success',
                'redirect' => $order->get_checkout_payment_url(true)
            );
        }

        /**
         * Output for the order received page.
         * */
        public function receipt_jcc_payment_gateway($order_id) {
            $order = wc_get_order($order_id);
         
            $data_array = array(
                'order_id' => $order->get_order_key(),
                'purchase_amt' => $order->get_total(),
            );
            try {
                $wc_jcc_api = $this->get_wc_jcc_api_instance($data_array);

                if ($wc_jcc_api->get_test_mode()) {
                    $request_url = $wc_jcc_api->get_test_request_url();
                    $merchant_id = $wc_jcc_api->get_test_merchant_id();
                } else {
                    $request_url = $wc_jcc_api->get_production_request_url();
                    $merchant_id = $wc_jcc_api->get_production_merchant_id();
                }

                echo '<div>
                <form method="post" name="paymentForm" id="paymentForm" action="' . $request_url . '">
                    <input type="hidden" name="Version" value="' . $wc_jcc_api->get_version() . '"><br>
                    <input type="hidden" name="MerID" value="' . $merchant_id . '"><br>
                    <input type="hidden" name="AcqID" value="' . $wc_jcc_api->get_acquirer_id() . '"><br>
                    <input type="hidden" name="MerRespURL" value="' . $wc_jcc_api->get_payment_gateway_response_url() . '"><br>
                    <input type="hidden" name="PurchaseAmt" value="' . $wc_jcc_api->get_formatted_purchase_amt() . '"><br>
                    <input type="hidden" name="PurchaseCurrency" value="' . $wc_jcc_api->get_currency_iso_code() . '"><br>
                    <input type="hidden" name="PurchaseCurrencyExponent" value="' . $wc_jcc_api->get_currency_exp() . '"><br> 
                    <input type="hidden" name="OrderID" value="' . $wc_jcc_api->get_order_id() . '"><br>
                    <input type="hidden" name="CaptureFlag" value="' . $wc_jcc_api->get_capture_flag() . '"><br>
                    <input type="hidden" name="Signature" value="' . $wc_jcc_api->get_base_64_signature() . '"><br> 
                    <input type="hidden" name="SignatureMethod" value="' . $wc_jcc_api->get_signature_method() . '"><br>
                </form>
                <script language="JavaScript">document.forms["paymentForm"].submit();</script>
                </div>';
            } catch (Exception $exception) {
                $notice_type = 'error';
                wc_add_notice($exception->getMessage(), $notice_type);
                return new WP_Error($exception->getMessage());
            }
        }

        public function jcc_payment_gateway_response() {
          
            //Parameters returned from JCC
            $data_array = array(
                'acquirer_id' => $_POST['AcqID'],
                'order_id' => $_POST['OrderID'], // corresponds to WC's order_key
                'response_code' => intval($_POST['ResponseCode']),
                'reason_code' => intval($_POST['ReasonCode']),
                'reason_code_description' => $_POST['ReasonCodeDesc'],
                'reference_number' => $_POST['ReferenceNo'],
                'padded_card_number' => $_POST['PaddedCardNo'],
                'response_signature' => $_POST['ResponseSignature'],
                'authorization_code' => $_POST['ResponseSignature'],
            );

            $test_mode = $this->test_mode === 'yes' ? true : false;

            if ($test_mode) {
                $data_array['test_merchant_id'] = $_POST['MerID'];
            } else {
                $data_array['production_merchant_id'] = $_POST['MerID'];
            }
            
            $wc_order_key = $data_array['order_id'];
            $order_id = wc_get_order_id_by_order_key($wc_order_key);
            $order = wc_get_order($order_id);

            $note = sprintf(__('JCC payment gateway response payload.<br> %s', 'woocommerce_jcc_payment_gateway'), print_r($data_array, true));
            
            //Add Admin Order Note
            $order->add_order_note($note);

            $order->update_meta_data('jcc_merchant_id', $_POST['MerID']);
            $order->update_meta_data('jcc_order_id', $wc_order_key);
            $order->update_meta_data('jcc_response_code', $data_array['response_code']);
            $order->update_meta_data('jcc_reason_code', $data_array['reason_code']);
            $order->update_meta_data('jcc_reason_code_description', $data_array['reason_code_description']);
            $order->update_meta_data('jcc_reference_no', $data_array['reference_number']);

            try {
                $wc_jcc_api = $this->get_wc_jcc_api_instance($data_array);

                if ($wc_jcc_api->get_response_code() == 1 && $wc_jcc_api->get_reason_code() == 1) {
                    // Success
                    $notice = sprintf(__('Payment via JCC Payment Gateway is successful (Order ID: %s)', 'woocommerce_jcc_payment_gateway'), $wc_order_key);

                    $order->payment_complete();

                    //Add Admin Order Note
                    $order->add_order_note($notice);

                    wc_empty_cart();

                    wp_redirect($this->get_return_url($order));
                    exit;
                } else {
                    // Failed
                    $notice = sprintf(__('Order ID: (%s) failed due to "%s".', 'woocommerce_jcc_payment_gateway'), $wc_order_key, $data_array['reason_code_description']);
                    
                    $order->update_status('failed', $notice);

                    //Add Admin Order Note
                    $notice_type = 'error';
                    wc_add_notice($notice, $notice_type);

                    wp_redirect(wc_get_page_permalink('checkout'));
                    exit;
                }
            } catch (Exception $exception) {
                $notice_type = 'error';
                wc_add_notice($exception->getMessage(), $notice_type);
                return new WP_Error($exception->getMessage());
            }
        }

        public function process_refund($order_id, $amount = null, $reason = '') {

            $order = wc_get_order($order_id);

            if (!$amount) {
                $amount = $order->get_total();
            }

            $order_key = $order->get_order_key();
            try {
                $wc_jcc_api = $this->get_wc_jcc_api_instance();
                
                $jcc_financial_service_response = $wc_jcc_api->do_refund($order_key, $amount, $reason);
                //syslog(LOG_INFO, 'process_refund::: jcc_financial_service_response: ' . print_r($jcc_financial_service_response, true));


                $refundAmountResult = $jcc_financial_service_response->RefundAmountResult; // boolean
                $sError = $jcc_financial_service_response->sError;
                
                $order->update_meta_data('jcc_refund_amount', $refundAmountResult);
                $order->update_meta_data('jcc_refund_error', $sError);

                if ($jcc_financial_service_response) {

                    if ($refundAmountResult == true && $sError == 'Success') {
                        $notice = sprintf(__('Order ID (%s) and key (%s) with the amount of %s has been successfully refunded.', 'woocommerce_jcc_payment_gateway'), $order_id, $order_key, $amount);
                        $order->add_order_note($notice);
                        $order->update_status('refunded', $notice);
                        return true;
                    } else {
                        $notice = sprintf(__('Order ID (%s) and key (%s) failed to be refunded due to %s. Please contact admininstrator for more help.', 'woocommerce_jcc_payment_gateway'), $order_id, $order_key, $sError);
                        $order->add_order_note($notice);
                        return new WP_Error('wc_' . $this->id . '_refund_failed', $notice);
                    }
                } else {
                    $notice = sprintf(__('Order ID (%s) and key (%s) failed to be refunded. Please contact admininstrator for more help.', 'woocommerce_jcc_payment_gateway'), $order_id, $order_key);
                    $order->add_order_note($notice);
                    return new WP_Error('wc_' . $this->id . '_refund_failed', $notice);
                }
            } catch (Exception $exception) {
                $notice = sprintf(__('Order ID (%s) failed to be refunded. Please contact admininstrator for more help.', 'woocommerce_jcc_payment_gateway'), $order_id);
                $notice_type = 'error';
                wc_add_notice($notice, $notice_type);
                return new WP_Error('wc_' . $this->id . '_refund_amount_failed_due_to', $exception->getMessage());
            }


            return false;
        }

    }
    
endif;